"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = require("react");

const useSocketHandler = onMessage => (0, _react.useCallback)(evt => {
  if (evt && evt.data) {
    onMessage(JSON.parse(evt.data));
  }
}, [onMessage]);

const useSocket = (url, {
  onMessage
} = {}) => {
  const messageQueue = (0, _react.useRef)([]);
  const socket = (0, _react.useRef)(null);
  const messageHandler = useSocketHandler(onMessage);
  const send = (0, _react.useCallback)(message => {
    if (socket.current.readyState === WebSocket.OPEN) {
      socket.current.send(JSON.stringify(message));
    } else {
      messageQueue.current.push(message);
    }
  }, []);
  (0, _react.useEffect)(() => {
    socket.current = new WebSocket(url);
    socket.current.addEventListener("open", () => {
      while (messageQueue.current.length > 0) {
        // send any queued messages on the socket now that it is open
        var msg = messageQueue.current.shift();
        send(msg);
      }
    });
    return () => socket.current && socket.current.close();
  }, [send, url]);
  (0, _react.useEffect)(() => {
    if (onMessage && socket.current) {
      socket.current.addEventListener("message", messageHandler);
    }

    return () => {
      if (onMessage && socket) {
        socket.current.removeEventListener("message", messageHandler);
      }
    };
  }, [messageHandler, onMessage]);
  return {
    send
  };
};

var _default = useSocket;
exports.default = _default;